/*
 * Hid_example_firmware.c
 *
 * Created: 22.06.2012 20:21:31
 *  Author: Flop
 */ 

#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>   /*   usbdrv.h */
#include "usbdrv.h"
#include "uart.h"

//#include "oddebug.h" /*      UART */

//    / / UART   
//  /
FILE uart_file = FDEV_SETUP_STREAM(uart_put, uart_get, _FDEV_SETUP_RW);


struct dataexchange_t       //     
{
   uchar b1;        //        3 .
   uchar b2;        //       PORTB.  
   uchar b3;        //   (  3  ).
};                  //       .
                    //        :)


struct dataexchange_t pdata = {0, 0, 0};


/*const PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = { // USB report descriptor         //       
    0x06, 0x00, 0xff,                       // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                             // USAGE (Vendor Usage 1)
    0xa1, 0x01,                             // COLLECTION (Application)
    0x15, 0x00,                             //    LOGICAL_MINIMUM (0)        // min.   
    0x26, 0xff, 0x00,                       //    LOGICAL_MAXIMUM (255)      // max.   , 255   ,     1 
    0x75, 0x08,                             //    REPORT_SIZE (8)            //   ,    "" 8 
    0x95, sizeof(struct dataexchange_t),    //    REPORT_COUNT               //   (   = 3,       )
    0x09, 0x00,                             //    USAGE (Undefined)
    0xb2, 0x02, 0x01,                       //    FEATURE (Data,Var,Abs,Buf)
    0xc0                                    // END_COLLECTION
};*/
/*      report, -     report-ID (    ).
 *     3   (  REPORT_SIZE = 8  = 1 ,   REPORT_COUNT = 3).
 */

const PROGMEM char usbHidReportDescriptor[USB_CFG_HID_REPORT_DESCRIPTOR_LENGTH] = {    /* USB report descriptor */
0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
0x09, 0x01,                    // USAGE (Vendor Usage 1)
0xa1, 0x01,                    // COLLECTION (Application)
0x85, 0x77,                     //   REPORT_ID (0x77)
0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
0x75, 0x08,                    //   REPORT_SIZE (8)
0x95, 0x80,                    //   REPORT_COUNT (128)
0x09, 0x00,                    //   USAGE (Undefined)
0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
//  INPUT 
0x09, 0x01,                    //   USAGE (Vendor Usage 1)
0x95, 0x05,                     //   REPORT_COUNT (5 bytes)
0x81, 0x02,                     //   INPUT (Data,Var,Abs)

0xc0                           // END_COLLECTION
};



/*       */
static uchar    currentAddress;
static uchar    bytesRemaining;


uint16_t isInt1 = 0;
uint16_t tim0_ovf_cnt;//   0,    


/* usbFunctionRead()        
 *    .   usbdrv.h
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
	printf_P(PSTR("usbFunctionRead()\n\r"));
    if(len > bytesRemaining)
        len = bytesRemaining;

    uchar *buffer = (uchar*)&pdata;

    if(!currentAddress)        //       .
    {                          //    
        if ( PINB & _BV(1) )
            pdata.b1 = 1;
        else
            pdata.b1 = 0;


        if ( PINB & _BV(2) )
            pdata.b2 = 1;
        else
            pdata.b2 = 0;


        if ( PINB & _BV(3) )
            pdata.b3 = 1;
        else
            pdata.b3 = 0;
    }

    uchar j;
    for(j=0; j<len; j++)
        data[j] = buffer[j+currentAddress];

    currentAddress += len;
    bytesRemaining -= len;
    return len;
}


/* usbFunctionWrite()        
 *    .   usbdrv.h
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
	printf_P(PSTR("usbFunctionWrite()\n\r"));
    if(bytesRemaining == 0)
        return 1;               /*   */

    if(len > bytesRemaining)
        len = bytesRemaining;

    uchar *buffer = (uchar*)&pdata;
    
    uchar j;
    for(j=0; j<len; j++)
        buffer[j+currentAddress] = data[j];

    currentAddress += len;
    bytesRemaining -= len;

    if(bytesRemaining == 0)     //   
    {                           //    PORTB
        if ( pdata.b1 )
            PORTB |= _BV(1);
        else
            PORTB &= ~_BV(1);


        if ( pdata.b2 )
            PORTB |= _BV(2);
        else
            PORTB &= ~_BV(2);


        if ( pdata.b3 )
            PORTB |= _BV(3);
        else
            PORTB &= ~_BV(3);
    }

    return bytesRemaining == 0; /* 0 ,     */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
	printf_P(PSTR("usbFunctionSetup()\n\r"));
	usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID  */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            //      ,   report-ID
            bytesRemaining = sizeof(struct dataexchange_t);
            currentAddress = 0;
            return USB_NO_MSG;  //  usbFunctionRead()    
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            //      ,   report-ID
            bytesRemaining = sizeof(struct dataexchange_t);
            currentAddress = 0;
            return USB_NO_MSG;  //  usbFunctionWrite()     
        }
    }else{
        /*      */
    }
    return 0;
}
/* ------------------------------------------------------------------------- */


void init_devices(void)
{
	DDRB = 0b00001110;      // PB1,PB2,PB3 - 
		
	// Port D initialization
	// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In
	// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T
	PORTD=0x00;
	DDRD=0x00;
	
	// Timer/Counter 0 initialization
	// Clock source: System Clock
	// Clock value: 250,000 kHz
	// Mode: Normal top=FFh
	// OC0 output: Disconnected
	TCCR0=0x03;
	TCNT0=0x00;
	OCR0=0x00;
	
	// External Interrupt(s) initialization
	// INT0: Off
	// INT1: On
	// INT1 Mode: Any change
	// INT2: Off
	GICR|=0x80;
	MCUCR=0x04;
	MCUCSR=0x00;
	GIFR=0x80;
	
	// Timer(s)/Counter(s) Interrupt(s) initialization
	TIMSK=0x01;
}


uint16_t cnt = 0;
uint16_t sigArr[100];
uint16_t testArr[100];
unsigned char isTimeout = 0;
unsigned char recvBuf[4];

//      :
#define B(bit_no)         (1 << (bit_no))//  
#define CB(reg, bit_no)   (reg) &= ~B(bit_no)//
#define SB(reg, bit_no)   (reg) |= B(bit_no)//
#define VB(reg, bit_no)   ( (reg) & B(bit_no) )//
#define TB(reg, bit_no)   (reg) ^= B(bit_no)//

#define zero_bit 0
#define one_bit 1
#define error_bit 255
#define start_bit 254
#define repeat_bit 253

#define PULT_GR 1
#define PULT_NONE 0
#define PULT_PRESS_KEY 1
#define PULT_REPEAT_KEY 2

uint16_t pulseTime;
uint16_t sigTime = 0;
unsigned char pult_a1 = 0;
unsigned char pult_a2 = 0;
unsigned char pult_c1 = 0;
unsigned char pult_c2 = 0;
unsigned char pult_state = PULT_NONE;
unsigned char pult_bit;

//    Timer0
ISR(TIMER0_OVF_vect)
{
	tim0_ovf_cnt++;
	if ((tim0_ovf_cnt > 195) && (cnt > 0))//,   IR     200  
	{
		isTimeout = 1;
		cnt = 0;
		sigTime = 0;
	}
}



//   INT1
ISR(SIG_INTERRUPT1)
{
	pulseTime = tim0_ovf_cnt * 256 + TCNT0;
	if (VB(PIND, 3) > 0)
	{
		sigTime = pulseTime;
	}
	else
	{
		if ((cnt < 100) && (sigTime > 0))
		{
			pult_bit = error_bit;
			if ((sigTime > 2100) && (sigTime < 2500) && (pulseTime > 1000) && (pulseTime < 1200))// 
			{
				cnt = 0;
				pult_bit = start_bit;
				pult_state = PULT_NONE;
			}
			else if ((sigTime > 100) && (sigTime < 200))
			{
				if ((pulseTime > 350) && (pulseTime < 600))
				{
					pult_bit = one_bit;
				}
				else if ((pulseTime > 100) && (pulseTime < 200))
				{
					pult_bit = zero_bit;
				}
				else if ((pulseTime > 23000) && (pulseTime < 25000))
				{
					pult_bit = repeat_bit;
					pult_state = PULT_REPEAT_KEY;
					SendPacketToUSB(1, 3);
				}
			}				
				
			sigArr[cnt] = sigTime;
			testArr[cnt] = pulseTime;
				
			if ((pult_bit == one_bit) || (pult_bit == zero_bit))
			{
				if ((cnt >= 1) && (cnt <= 8))
				{
					pult_a1 = (pult_a1 << 1);
					pult_a1 = (pult_a1 | pult_bit);
				}
				else if ((cnt >= 9) && (cnt <= 16))
				{
					pult_a2 = (pult_a2 << 1);
					pult_a2 = (pult_a2 | pult_bit);
				}
				else if ((cnt >= 17) && (cnt <= 24))
				{
					pult_c1 = (pult_c1 << 1);
					pult_c1 = (pult_c1 | pult_bit);
				}
				else if ((cnt >= 25) && (cnt <= 32))
				{
					pult_c2 = (pult_c2 << 1);
					pult_c2 = (pult_c2 | pult_bit);
				}
				if (cnt == 32)//  
				{
					pult_state = PULT_PRESS_KEY;
					SendPacketToUSB(1, 3);
				}					
			}
			if (isTimeout == 0)
			{
				cnt++;
			}
			else
			{
				cnt = 0;
				isTimeout = 0;
			}
		}
		else
		{
			if (cnt > 100)
				printf_P(PSTR("cnt > 100\n\r"));
		}
	}
		
	// 
	tim0_ovf_cnt = 0;
	TCNT0 = 0;
}	

void SendPacketToUSB(unsigned char sendTo, unsigned char bytesOfWrite)
{
	if (usbInterruptIsReady())
	{
		//printf_P(PSTR("Send data to HID\n\r"));
		unsigned char buf[6];
		buf[0]= 0x77; //ReportID
		buf[1]= sendTo;
		buf[2]= bytesOfWrite;
		buf[3]= 1;
		buf[4]= pult_state;
		buf[5]= pult_c1;
		usbSetInterrupt(buf, 6);
	}
	else
	{
		printf_P(PSTR("usb Interrupt not Ready\n\r"));
	}
}


int main(void)
{
	init_devices();

	uart_init(); //  -    
	stdout = stdin = &uart_file; //       

    usbInit();
    usbDeviceDisconnect();  //    ,       !
    
    uchar i = 0;
    while(--i){             //  > 250 ms
        _delay_ms(1);
    }
    

    usbDeviceConnect();     // 

    sei();                  //  
	//GIMSK |= (1<<INT1);  //  INT1  

	printf_P(PSTR("Program Start\n\r"));
	unsigned char buf[6];
	
	//for(int i; i < 10; i++)
	{
		//  
		for(int i = 0; i < 100; i++)
		{
			usbPoll();          //        ,     - 50 ms
			_delay_ms(20);
		}
		
		if (usbInterruptIsReady())
		{
			printf_P(PSTR("Send data to HID\n\r"));
			buf[0]=0x77; //ReportID
			buf[1]='K';
			buf[2]='e';
			buf[3]='y';
			buf[4]='0';
			buf[5]= 0;
			usbSetInterrupt(buf, 6);
		}
		else
		{
			printf_P(PSTR("usb Interrupt not Ready\n\r"));
		}
	}
	printf_P(PSTR("start cycle\n\r"));
	for(;;)
	{                //   
        usbPoll();          //        ,     - 50 ms
		_delay_ms(20);
		//printf_P(PSTR("tim0_ovf_cnt: %u\n\r"), tim0_ovf_cnt);
		//_delay_ms(2000);
		
		if (isTimeout > 0)
		{
			/*printf_P(PSTR("----------------\n\r"));
			for (i = 0; i < 100, (testArr[i] > 0)||(sigArr[i] > 0); i++)
			{	
				_delay_ms(100);
				printf_P(PSTR("%u: %u   %u\n\r"), i, sigArr[i], testArr[i]);
				sigArr[i] = 0;
				testArr[i] = 0;
			}*/
			//_delay_ms(100);
			printf_P(PSTR("a1: %u\n\r"), pult_a1);
			//_delay_ms(100);
			printf_P(PSTR("a2: %u\n\r"), pult_a2);
			//_delay_ms(100);
			printf_P(PSTR("c1: %u\n\r"), pult_c1);
			//_delay_ms(100);
			printf_P(PSTR("c2: %u\n\r"), pult_c2);
			//_delay_ms(100);
			/*if (pult_state == PULT_PRESS_KEY)
				printf_P(PSTR("PRESS\n\r"));
			if (pult_state == PULT_REPEAT_KEY)
				printf_P(PSTR("REPEAT\n\r"));
				
			_delay_ms(200);*/
			isTimeout = 0;
			isInt1 = 0;
		}
		//if (VB(PIND, 3) == 0)
			//printf_P(PSTR("+"));	
    }
    return 0;
}
/* ------------------------------------------------------------------------- */ 